import com.dalsemi.comm.*;
import com.dalsemi.system.*;
import java.io.*;

public class CanBusInterface extends Thread
{
  static boolean bStopThread;
  static int nRxCanMsgs;
  static CanFrame lastFrame = new CanFrame();
  static CanBus a = null;

  // 125 Kbit/s with crystal of 18.432MHz
//  static final int CAN_DIVISOR = 7;
//  static final int CAN_TSEG1 = 13;
//  static final int CAN_TSEG2 = 7;
//  static final int CAN_SJW = 1;

  // 246 Kbit/s with crystal of 18.432MHz
//  static final int CAN_DIVISOR = 5;
//  static final int CAN_TSEG1 = 9;
//  static final int CAN_TSEG2 = 5;
//  static final int CAN_SJW = 2;

  // 250 Kbit/s with crystal of 20 MHz
  static final int CAN_DIVISOR = 5;
  static final int CAN_TSEG1 = 10;
  static final int CAN_TSEG2 = 5;
  static final int CAN_SJW = 2;
  static final int CAN_SAMPLES = 1;

  static final byte CANBUSNUM = CanBus.CANBUS0;
  static final boolean dopassive = false;

  public CanBusInterface()
  {
    try
    {
      a = new CanBus(CANBUSNUM);

      a.setBaudRatePrescaler(CAN_DIVISOR);
      a.setTSEG1(CAN_TSEG1);
      a.setTSEG2(CAN_TSEG2);
      a.setSynchronizationJumpWidth(CAN_SJW);
      a.setSampleRate(CAN_SAMPLES);

      a.setReceiveQueueLimit(1);
      a.setTransmitQueueLimit(1);
      System.out.println("Frames available to read: "+a.receiveFramesAvailable());

      // Now, we tell the CAN Controller to jump on the bus.
      if (dopassive)
      {
        System.out.println("Enabling passive receive");
        a.enableControllerPassive();
      }
      else
      {
        System.out.println("Enabling regular receive");
        a.enableController();
      }

      // Set message center one to transmit.  This allows any outgoing messages
      // to use this register.
      a.setMessageCenterTXMode(1);
      a.enableMessageCenter(1);
    }
    catch (Throwable e)
    {
      System.out.println(e);
    }
  }

  static void dumpFrame(CanFrame frame)
  {
    if (frame.extendedID)
      System.out.print("Ext ID: ");
    else
      System.out.print("Std ID: ");
    System.out.println(Integer.toHexString(frame.ID));
    System.out.println("Length: " + frame.length);
    for (int i=0; i<frame.length; i++)
      System.out.print(Integer.toHexString(frame.data[i] & 0xFF) + " ");
    System.out.println();
  }

  static void initialize() throws Exception
  {
    final int DUMPCOUNT = 1;
    long start,stop;

    nRxCanMsgs = 0;

    // Set message center 2 to receive
    a.setMessageCenterRXMode(2);

    // Set message center one to use mask filtering
    a.set29BitGlobalIDMask(0);
    a.setMessageCenterMessageIDMaskEnable(2,true);

    // Set message center 2 to match this 29 bit extended id
//    a.set29BitMessageCenterArbitrationID(2,0x55F6575C);

    // Set message center 2 to allow reception of messages.
    a.setMessageCenterWriteOverEnable(2,true);
    a.enableMessageCenter(2);

    bStopThread = false;
    while (!bStopThread)
    {
      // Block waiting on a frame reception.
      a.receive(lastFrame);
//	timeStamp = TINIOS.uptimeMillis();

      nRxCanMsgs++;
//      dumpFrame(lastFrame);
    }
    a.close();
  }

  public int getRxCanMsgCount()
  {
    return nRxCanMsgs;
  }

  public void writeLastCanFrame(DataOutputStream dsOut)
  {
    try
    {
      dsOut.writeBoolean(lastFrame.extendedID);
      dsOut.writeInt(lastFrame.ID);
      dsOut.writeInt(lastFrame.length);
      dsOut.write(lastFrame.data,0,lastFrame.length);
    }
    catch (java.io.IOException e)
    {
      System.out.print("test out ");
      System.out.println(e);
    }
  }

  public void sendCanFrame(CanFrame cf)
  {
    try
    {
      // Send a frame using extended (29 bit) ID, block until frame is ACKed
      a.sendFrame(cf);
    }
    catch (Throwable e)
    {
      System.out.println(e);
    }
  }

  public void stopCmd()
  {
    bStopThread = true;
  }

  public void run()
  {
    try
    {
      System.out.println("CAN BUS Interface starts ...");
      initialize();
      System.out.println("CAN BUS Interface normal exit");
    }
    catch (Throwable e)
    {
      System.out.println("CAN BUS Interface exception");
      System.out.println(e);
    }
  }
}
