/*
 * CanFramesClient.java
 */

/*
 $Workfile: CanFramesClient.java $
 $Revision: 0 $
 $Date: 2000/06/05 $
 $Author: Jaap van Bergeijk $
 $Modtime: 2000/10/30 $
 $Log: /java/project/CanMonitor/Server/src/CanFramesClient.java $
*
*/

import java.io.*;
import java.net.*;
import com.dalsemi.comm.*;

/** CanFramesClient forwards CAN frames from Ethernet to the CAN bus.
  */
class CanFramesClient extends Thread
{
  private Socket framesSocket;
  private CanBusInterface can;

  private boolean bStopThread = true;

/** Constructor
  */
  CanFramesClient(Socket s, CanBusInterface c)
    throws IOException
  {
    this.framesSocket = s;
    this.can = c;
  }

/** Forward received CanFrames until connection is closed.
  */
  public void run()
  {
    DataInputStream dIn;

    try
    {
      dIn = new DataInputStream(framesSocket.getInputStream());

      System.out.println("Frames client thread started");

      CanFrame cf = new CanFrame();
      int itemNumber = 0;
      int bytesToRead = 0;
      cf.extendedID = true;
      cf.ID = 0;
      cf.length = 0;

      try
      {
        bStopThread = false;
        while (!bStopThread)
        {
          switch (itemNumber)
          {
          case 0:
            cf.extendedID = dIn.readBoolean();
            itemNumber++;
            break;
          case 1:
            cf.ID = dIn.readInt();
            itemNumber++;
            break;
          case 2:
            cf.length = dIn.readInt();
            bytesToRead = cf.length;
            itemNumber++;
            break;
          case 3:
            if (bytesToRead > 0)
            {
              cf.data[cf.length-bytesToRead] = dIn.readByte();
              bytesToRead--;
            }
            else
            {
              can.sendCanFrame(cf);
              itemNumber = 0;
            }
            break;
          }
        }
      }
      catch (IOException ioe)
      {
        System.out.println(ioe + " frames client 1");
      }
      finally
      {
        dIn.close();
      }
    }
    catch (IOException ioe)
    {
      System.out.println(ioe + " frames client 2");
    }
    finally
    {
      System.out.println("Frames client thread stopped");
    }
  }
}
