/*
 * CanFramesServer.java
 */

/*
 $Workfile: CanFramesConnection.java $
 $Revision: 0 $
 $Date: 2000/06/05 $
 $Author: Jaap van Bergeijk $
 $Modtime: 2000/10/30 $
 $Log: /java/project/CanMonitor/Server/src/CanFramesConnection.java $
*
*/

import java.io.*;
import java.net.*;

/** CanFramesConnection opens a Socket to a CanMonitor and
  * forwards CAN frames from the CAN bus over Ethernet to the CanMonitor.
  * Creates a CanFramesClient to forward CanFrames from Ethernet to the
  * CAN bus.
  */
class CanFramesConnection extends Thread
{
  private String serverName;
  private int serverPort;
  private CanBusInterface can;

  private boolean bStopThread = true;
  private boolean bConnected = false;

/** Constructor
  */
  CanFramesConnection(CanBusInterface can)
    throws IOException
  {
    this.can = can;
  }

/** Set Server Name and Server Port to use for the socket connection.
  */
  public void setServer(String serverName, int serverPort)
  {
    this.serverName = serverName;
    this.serverPort = serverPort;
  }

/** Forward CAN messages until connection is closed by client.
  */
  public void run()
  {
    Socket framesSocket;
    DataOutputStream dOut;
    CanFramesClient cfc;
    int nRxCanMsgs;

    try
    {
      // try to open Frames Communication Socket
      framesSocket = new Socket(InetAddress.getByName(serverName), serverPort);
      bConnected = true;

      try
      {
        // start a new thread to forward CAN frames to the CAN bus
        cfc = new CanFramesClient(framesSocket, can);
        cfc.start();

        try
        {
          // open output stream to forward CAN frames over Ethernet
          dOut = new DataOutputStream(framesSocket.getOutputStream());

          // get current message count
          nRxCanMsgs = can.getRxCanMsgCount();

          System.out.println("Frames connection thread started, message count: " + nRxCanMsgs);

          bStopThread = false;
          while (!bStopThread)
          {
            // check for new messages
            if (nRxCanMsgs != can.getRxCanMsgCount())
            {
              nRxCanMsgs = can.getRxCanMsgCount();
              can.writeLastCanFrame(dOut);
            }
          }

          System.out.println("Frames connection thread stopped");
          bConnected = false;
          dOut.close();
        }
        catch(IOException ioe)
        {
          System.out.println(ioe + " frames connection 1");
        }

        cfc.stop();
      }
      catch(IOException ioe)
      {
        System.out.println(ioe + " frames connection 2");
      }

      try
      {
        framesSocket.close();
      }
      catch (IOException ioe)
      {
        System.out.println(ioe + " frames connection 3");
      }
    }
    catch (IOException ioe)
    {
      System.out.println(ioe + " frames connection 4");
    }
  }

/** Stop CAN frames connection.
  */
  public void stopThread()
  {
    bStopThread = true;
  }

/** returns state of the CAN frames connection.
  */
  public boolean isConnected()
  {
    return bConnected;
  }
}
