/*
 $Workfile: CanMonitorServer.java $
 $Revision: 0 $
 $Date: 2000/06/05 $
 $Author: Jaap van Bergeijk $
 $Modtime: 2000/10/30 $
 $Log: /java/project/CanMonitor/Server/src/CanMonitorServer.java $
*/

import java.io.*;
import java.net.*;

class CanMonitorServer extends Thread
{
  static final int STATUS_PORT = 7;
  static final int FRAMES_PORT = 8;
  static final int MAX_CONNECTIONS = 3;
  static private CanBusInterface canBus;
  static private int nConnections;
  private Socket statusSocket;
  private CanStatusClient statusClient;
  private CanFramesConnection framesConnection;
  private int nRxCanMsgs;
  boolean bStopThread = true;

/** Constructor
  */
  CanMonitorServer(Socket s) throws IOException
  {
    this.statusSocket = s;
    framesConnection = new CanFramesConnection(canBus);
  }

/** Send CAN status until connection is closed by client.
  */
  public void run()
  {
    nConnections++;
    System.out.println("New Connection: " + nConnections);

    try
    {
      statusClient = new CanStatusClient(this, statusSocket,canBus);
      statusClient.start();
    }
    catch (IOException ioe) {}

    String str = new String();
    nRxCanMsgs = canBus.getRxCanMsgCount();
    str = "Current: " + nRxCanMsgs + "\n";
    statusClient.write(str.getBytes());

    bStopThread = false;
    while (!bStopThread)
    {
      if (nRxCanMsgs != canBus.getRxCanMsgCount())
      {
        nRxCanMsgs = canBus.getRxCanMsgCount();
        // do something for status forwarding
        // ....
      }
    }

    // disconnect
    nConnections--;
    closeFramesConnection();
    try
    {
      statusClient.stopThread();
      System.out.println("Closing: " + statusSocket);
      statusSocket.close();
    }
    catch (IOException i) {}
  }

  public void stopThread()
  {
    bStopThread = true;
  }

  public void openFramesConnection()
  {
    if (!framesConnection.isConnected())
    {
      framesConnection.setServer(statusSocket.getInetAddress().getHostName(), FRAMES_PORT);
      framesConnection.start();
    }
  }

  public void closeFramesConnection()
  {
    if (framesConnection.isConnected()) framesConnection.stopThread();
  }

/** main - Opens a server socket and spins off a new thread each time
  *        a new client connection is accepted on this socket.
  */
  public static void main(String[] args)
  {
    System.out.println("Starting CanMonitorServer version 1.0 ...");
    nConnections = 0;
    ServerSocket serverSocket = null;

    try
    {
      serverSocket = new ServerSocket(STATUS_PORT);

      // start a CAN bus interface
      canBus = new CanBusInterface();
      canBus.start();

      // wait for new clients
      while (true)
      {
        if (nConnections < MAX_CONNECTIONS) (new CanMonitorServer(serverSocket.accept())).start();
      }
    }
    catch(Exception e) {}
    finally
    {
      try
      {
        serverSocket.close();
        canBus.stopCmd();
      }
      catch (IOException i)
      {
        System.out.println(i);
      }
    }
  }
}
