/*
 $Workfile: CanStatusClient.java $
 $Revision: 0 $
 $Date: 2000/06/05 $
 $Author: Jaap van Bergeijk $
 $Modtime: 2000/10/30 $
 $Log: /java/project/CanMonitor/Server/src/CanStatusClient.java $
*
*/

import java.io.*;
import java.net.*;
import com.dalsemi.comm.*;

/** CanStatusClient wait for Status messages from a Client.
  */
class CanStatusClient extends Thread
{
  private CanMonitorServer m;
  private Socket s;
  private InputStream sIn = null;
  private OutputStream sOut = null;
  private CanBusInterface can;
  private int nRxCanMsgs;
  private boolean bConnected = false;
  private boolean bStopThread = true;

/** Constructor
  */
  CanStatusClient(CanMonitorServer m, Socket s, CanBusInterface can)
    throws IOException
  {
    this.m = m;
    this.s = s;
    this.can = can;
  }

/** Echo received data until connection is closed by client.
  */
  public void run()
  {
    System.out.println("CAN status client thread started");

    try
    {
      sIn  = s.getInputStream();
      sOut  = s.getOutputStream();

      bConnected = true;
      bStopThread = false;
      while (!bStopThread)
      {
        // blocked read
        byte b = (byte) sIn.read();
        if (b == 'd')
        {
          System.out.println("CAN status thread stop command");
          bStopThread = true;
        }
        if (b == 'o')
        {
          System.out.println("CAN status thread open frames connection command");
          m.openFramesConnection();
        }
        if (b == 'c')
        {
          System.out.println("CAN status thread close frames connection command");
          m.closeFramesConnection();
        }
      }
    }
    catch (IOException ioe)
    {
      System.out.println(ioe);
    }
    finally
    {
      try
      {
        bConnected = false;
        System.out.println("CAN status client thread stopped");
        sOut.close();
        sIn.close();
        m.stopThread();
      }
      catch (IOException i)
      {
        System.out.println(i);
      }
    }
  }

  public void stopThread()
  {
    bStopThread = true;
  }

  public void write(byte[] str)
  {
    if (bConnected)
    {
      try
      {
        sOut.write(str);
      }
      catch (IOException ioe)
      {
        System.out.println(ioe);
      }
    }
  }
}
